# Generated by Django 2.2.14 on 2020-07-15 14:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0004_auto_20190104_1242'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='reply_to_email',
            field=models.CharField(blank=True, max_length=200, verbose_name='reply to email'),
        ),
    ]
