{% extends email_html_theme %}
{% load aldryn_forms_tags i18n %}

{% block email_content %}
{% render_notification_text email_notification 'body_html' as html_message %}
{% if html_message %}
    {{ html_message|safe }}
{% else %}
    <p>{% blocktrans %}Form name: {{ form_name }}{% endblocktrans %}</p>
    {% include "aldryn_forms/emails/includes/form_data.html" %}
{% endif %}
{% endblock email_content %}
