# Generated by Django 2.2.16 on 2020-09-15 16:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0012_auto_20190104_1242'),
    ]

    operations = [
        migrations.AddField(
            model_name='formplugin',
            name='is_enable_autofill_from_url_params',
            field=models.BooleanField(default=False, help_text="Eg if you open the form with a url that contains parameters as 'https://example.com/sub-page/?email=hello@example.com&name=Alex', then the fields 'email' and 'name' are going to to be filled in automatically.", verbose_name='Enable autofill from url parameters'),
        ),
    ]
