# Generated by Django 1.10.8 on 2018-02-06 19:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0002_form_plugin_add_missing_permission'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailnotification',
            options={'verbose_name': 'Email notification', 'verbose_name_plural': 'Email notifications'},
        ),
    ]
