# Generated by Django 2.2.13 on 2022-03-18 08:16

from django.db import migrations

import aldryn_forms.sizefield.models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0014_multiple_files_pload'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
        migrations.AlterField(
            model_name='multiplefilesuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
    ]
