{% load i18n %}
{% filter linebreaksbr %}{% spaceless %}{% for field in data %}
{% trans "Field" %}: {{ field.name }}
{% trans "Label" %}: {{ field.label }}
{% trans "Value" %}: {{ field.value }}
{% endfor %}{% endspaceless %}
{% endfilter %}
